package es.rickyepoderi.keycloaksample;

import android.content.Intent;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.Key;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.Properties;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "MainActivity";

    private TokenResponse token;
    private Properties configuration;
    private Key key;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.token = null;
        loadConfiguration();
        ArrayList<TokenResponse> list = null;
        Intent intent = getIntent();
        if (savedInstanceState != null && savedInstanceState.containsKey("token-response")) {
            // first try with the saved token
            list = savedInstanceState.getParcelableArrayList("token-response");
        } else if (Intent.ACTION_VIEW.equals(intent.getAction())) {
            // check if we have to login
            Uri uri = intent.getData();
            if (uri != null) {
                String code = uri.getQueryParameter("code");
                if (code != null) {
                    new LoginTask(this,
                            new Callback<TokenResponse>() {
                                @Override
                                public void onPostExecute(MainActivity activity, TokenResponse tokenResponse) {
                                    activity.setToken(tokenResponse);
                                }
                            }
                    ).execute(code);
                }
            }
        }
        setContentView(R.layout.activity_main);
        if (list != null && !list.isEmpty()) {
            this.setToken(list.get(0));
        }
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        if (token != null) {
            outState.putParcelableArrayList("token-response", new ArrayList<>(Collections.singletonList(token)));
        } else {
            outState.remove("token-response");
        }
        super.onSaveInstanceState(outState);
    }

    private void loadConfiguration() {
        try {
            InputStream is = getAssets().open("configuration.properties");
            configuration = new Properties();
            configuration.load(is);
        } catch (IOException e) {
            Log.e(TAG, "Error loading configuration", e);
        }
    }

    public URL createTokenURL() throws MalformedURLException {
        // "http://192.168.100.1:8080/auth/realms/master/protocol/openid-connect/token"
        return new URL(new Uri.Builder().scheme(configuration.getProperty("keycloak.scheme"))
                .encodedAuthority(configuration.getProperty("keycloak.authority"))
                .appendEncodedPath("auth/realms")
                .appendPath(configuration.getProperty("keycloak.realm"))
                .appendEncodedPath("protocol/openid-connect/token")
                .build().toString());
    }

    public String createAuthorizationCodePostData(String code) {
        // "grant_type=authorization_code&client_id=KeycloakSample&redirect_uri=app%3A%2F%2Fkeycloaksample%2F&code=" + code
        return new Uri.Builder()
                .appendQueryParameter("grant_type", "authorization_code")
                .appendQueryParameter("client_id", getString(R.string.app_name))
                .appendQueryParameter("redirect_uri", configuration.getProperty("app.url"))
                .appendQueryParameter("code", code)
                .build().getEncodedQuery();
    }

    public String createRefreshPostData(String refreshToken) {
        // "grant_type=refresh_token&client_id=KeycloakSample&refresh_token=" + refreshToken
        return new Uri.Builder()
                .appendQueryParameter("grant_type", "refresh_token")
                .appendQueryParameter("client_id", getString(R.string.app_name))
                .appendQueryParameter("refresh_token", refreshToken)
                .build().getEncodedQuery();
    }

    public URL createUserInfoURL() throws MalformedURLException {
        // "http://192.168.100.1:8080/auth/realms/master/protocol/openid-connect/userinfo"
        return new URL(new Uri.Builder().scheme(configuration.getProperty("keycloak.scheme"))
                .encodedAuthority(configuration.getProperty("keycloak.authority"))
                .appendEncodedPath("auth/realms")
                .appendPath(configuration.getProperty("keycloak.realm"))
                .appendEncodedPath("protocol/openid-connect/userinfo")
                .build().toString());
    }

    public void login(View view) {
        // "http://192.168.100.1:8080/auth/realms/master/protocol/openid-connect/auth?client_id=KeycloakSample&scope=openid&response_type=code&redirect_uri=app%3A%2F%2Fkeycloaksample%2F";
        Uri uri = new Uri.Builder().scheme(configuration.getProperty("keycloak.scheme"))
                .encodedAuthority(configuration.getProperty("keycloak.authority"))
                .appendEncodedPath("auth/realms")
                .appendPath(configuration.getProperty("keycloak.realm"))
                .appendEncodedPath("protocol/openid-connect/auth")
                .appendQueryParameter("client_id", getString(R.string.app_name))
                .appendQueryParameter("scope", "openid")
                .appendQueryParameter("response_type", "code")
                .appendQueryParameter("redirect_uri", configuration.getProperty("app.url"))
                .build();
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);
    }

    public void logout(View view) {
        //String url = "http://192.168.100.1:8080/auth/realms/master/protocol/openid-connect/logout?redirect_uri=app%3A%2F%2Fkeycloaksample%2F";
        Uri uri = new Uri.Builder().scheme(configuration.getProperty("keycloak.scheme"))
                .encodedAuthority(configuration.getProperty("keycloak.authority"))
                .appendEncodedPath("auth/realms")
                .appendPath(configuration.getProperty("keycloak.realm"))
                .appendEncodedPath("protocol/openid-connect/logout")
                .appendQueryParameter("redirect_uri", configuration.getProperty("app.url"))
                .build();
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);
    }

    public void showAccessToken(View view) {
        if (this.token != null) {
            StringBuilder sb = new StringBuilder();
            for (Map.Entry<String,Object> e: token.getAccessTokenClaims().getBody().entrySet()) {
                sb.append(e.getKey()).append(": ").append(e.getValue()).append("\n");
            }
            sb.append("isExpired: ").append(token.isExpired()).append("\n");
            sb.append("isRefreshExpired: ").append(token.isRefreshExpired()).append("\n");
            this.setTextMain(sb.toString());
        } else {
            this.setTextMain("");
        }
    }

    public void refreshToken(View view) {
        if (this.token != null && this.token.isRefreshExpired()) {
            this.login(view);
        } else if (this.token != null) {
            new RefreshTask(this,
                    new Callback<TokenResponse>() {
                        @Override
                        public void onPostExecute(MainActivity activity, TokenResponse tokenResponse) {
                            activity.setToken(tokenResponse);
                        }
                    }
            ).execute(this.token.getRefreshToken());
        } else {
            this.setTextMain("");
        }
    }

    public void userInfo(View view) {
        if (this.token != null && this.token.isRefreshExpired()) {
            this.login(view);
        } else if (this.token != null && !token.isExpired()) {
            new UserInfoTask(this,
                    new Callback<String>() {
                        @Override
                        public void onPostExecute(MainActivity activity, String s) {
                            activity.setTextMain(s);
                        }
                    }
            ).execute(this.token.getAccessToken());
        } else if (this.token != null) {
            new RefreshTask(this,
                    new Callback<TokenResponse>() {
                        @Override
                        public void onPostExecute(MainActivity activity, TokenResponse tokenResponse) {
                            activity.setToken(tokenResponse);
                            new UserInfoTask(activity,
                                    new Callback<String>() {
                                        @Override
                                        public void onPostExecute(MainActivity activity, String s) {
                                            activity.setTextMain(s);
                                        }
                                    }
                            ).execute(activity.getToken().getAccessToken());
                        }
                    }
            ).execute(this.token.getRefreshToken());
        } else {
            this.setTextMain("");
        }
    }

    public void setToken(TokenResponse token) {
        if (!token.isError()) {
            this.token = token;
            setTextMain("");
            setLoggedIn(token.getAccessTokenClaims().getBody().get("preferred_username", String.class));
        } else {
            this.token = null;
            setTextMain(token.getError());
            setNotLoggedIn();
        }
        updateButtons();
    }

    public TokenResponse getToken() {
        return token;
    }

    public void updateButtons() {
        int modeOff = this.token == null? View.GONE : View.VISIBLE;
        int modeOn = this.token != null? View.GONE : View.VISIBLE;
        updateVisibility(R.id.loginButton, modeOn);
        updateVisibility(R.id.logoutButton, modeOff);
        updateVisibility(R.id.showTokenButton, modeOff);
        updateVisibility(R.id.refreshButton, modeOff);
        updateVisibility(R.id.userInfoButton, modeOff);
    }

    public void updateVisibility(int id, int mode) {
        findViewById(id).setVisibility(mode);
    }

    public void setTextMain(String value) {
        ((TextView) findViewById(R.id.textMain)).setText(value);
    }

    public void setLoggedIn(String name) {
        ((TextView) findViewById(R.id.textBottom)).setText(
                getResources().getString(R.string.logged_in, name));
    }

    public void setNotLoggedIn() {
        ((TextView) findViewById(R.id.textBottom)).setText(
                getResources().getString(R.string.not_logged_in));
    }

    public Properties getConfiguration() {
        return configuration;
    }

    public Key getSignatureKey() throws InvalidKeySpecException, NoSuchAlgorithmException {
        if (key == null) {
            byte[] byteKey = Base64.decode(configuration.getProperty("keycloak.key").getBytes(), Base64.DEFAULT);
            X509EncodedKeySpec X509publicKey = new X509EncodedKeySpec(byteKey);
            KeyFactory kf = KeyFactory.getInstance("RSA");
            key = kf.generatePublic(X509publicKey);
        }
        return key;
    }
}
