package es.rickyepoderi.keycloaksample;

import android.os.Parcel;
import android.os.Parcelable;
import android.util.JsonReader;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.security.Key;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.Jwts;

public class TokenResponse implements Parcelable {

    public static final int DEFAULT_SKEW = 5;
    private static final String TAG = "TokenResponse";

    private int expiresIn;
    private int refreshExpiresIn;
    private String accessToken;
    private Jws<Claims> accessTokenClaims;
    private String refreshToken;
    private Jws<Claims> refreshTokenClaims;
    private String errorMessage;
    private Key key;
    private long creationTime;

    public TokenResponse(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public TokenResponse(InputStream is, Key key) throws IOException {
        this.creationTime = System.currentTimeMillis() / 1000L;
        this.key = key;
        JsonReader jsonReader = new JsonReader(new InputStreamReader(is, "UTF-8"));
        jsonReader.beginObject();
        while(jsonReader.hasNext()) {
            String name = jsonReader.nextName();
            switch (name) {
                case "expires_in":
                    this.setExpiresIn(jsonReader.nextInt());
                    break;
                case "refresh_expires_in":
                    this.setRefreshExpiresIn(jsonReader.nextInt());
                    break;
                case "access_token":
                    this.setAccessToken(jsonReader.nextString());
                    break;
                case "refresh_token":
                    this.setRefreshToken(jsonReader.nextString());
                    break;
                default:
                    jsonReader.skipValue();
            }
        }
    }

    public TokenResponse(Parcel in) throws NoSuchAlgorithmException, InvalidKeySpecException {
        byte[] encoded = new byte[in.readInt()];
        in.readByteArray(encoded);
        this.key = KeyFactory.getInstance("RSA").generatePublic(new X509EncodedKeySpec(encoded));
        this.creationTime = in.readLong();
        this.setExpiresIn(in.readInt());
        this.setRefreshExpiresIn(in.readInt());
        this.setAccessToken(in.readString());
        this.setRefreshToken(in.readString());
    }

    public int getExpiresIn() {
        return expiresIn;
    }

    public void setExpiresIn(int expiresIn) {
        this.expiresIn = expiresIn;
    }

    public boolean isExpired() {
        return isExpired(DEFAULT_SKEW);
    }

    private boolean isExpired(int skew) {
        return (System.currentTimeMillis() / 1000L) + skew > creationTime + expiresIn;
    }

    public int getRefreshExpiresIn() {
        return refreshExpiresIn;
    }

    public void setRefreshExpiresIn(int refreshExpiresIn) {
        this.refreshExpiresIn = refreshExpiresIn;
    }

    public boolean isRefreshExpired() {
        return isRefreshExpired(DEFAULT_SKEW);
    }

    private boolean isRefreshExpired(int skew) {
        return (System.currentTimeMillis() / 1000L) + skew > creationTime + refreshExpiresIn;
    }

    public Jws<Claims> getAccessTokenClaims() {
        return accessTokenClaims;
    }

    public String getAccessToken() {
        return accessToken;
    }

    public void setAccessToken(String accessToken) {
        this.accessToken = accessToken;
        this.accessTokenClaims = Jwts.parser()
                .setSigningKey(this.key)
                .parseClaimsJws(accessToken);
    }

    public Jws<Claims> getRefreshTokenClaims() {
        return refreshTokenClaims;
    }

    public String getRefreshToken() {
        return refreshToken;
    }

    public void setRefreshToken(String refreshToken) {
        this.refreshToken = refreshToken;
        this.refreshTokenClaims = Jwts.parser()
                .setSigningKey(this.key)
                .parseClaimsJws(refreshToken);
    }

    public boolean isError() {
        return errorMessage != null;
    }

    public String getError() {
        return errorMessage;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel out, int flags) {
        out.writeInt(key.getEncoded().length);
        out.writeByteArray(key.getEncoded());
        out.writeLong(this.creationTime);
        out.writeInt(this.expiresIn);
        out.writeInt(this.refreshExpiresIn);
        out.writeString(accessToken);
        out.writeString(refreshToken);
    }

    public static final Parcelable.Creator<TokenResponse> CREATOR = new Parcelable.Creator<TokenResponse>() {
        public TokenResponse createFromParcel(Parcel in) {
            try {
                return new TokenResponse(in);
            } catch (NoSuchAlgorithmException|InvalidKeySpecException e) {
                Log.e(TAG, "Error creating the respose from the parcel", e);
                return null;
            }
        }

        public TokenResponse[] newArray(int size) {
            return new TokenResponse[size];
        }
    };
}
